@@ -167,6 +167,7 @@ class SeldonClient:
         call_credentials: SeldonCallCredentials = None,
         debug: bool = False,
         client_return_type: str = "dict",
+        ssl: bool = None,
     ):
         """
 
@@ -282,6 +283,8 @@ class SeldonClient:
         http_path: str = None,
         meta: Dict = None,
         client_return_type: str = None,
+        raw_data: Dict = None,
+        ssl: bool = None,
     ) -> SeldonClientPrediction:
         """
 
@@ -324,9 +327,11 @@ class SeldonClient:
         http_path:
            Custom http path for predict call to use
         meta:
-           Custom meta map
+           Custom meta map, supplied as tags
         client_return_type
             the return type of all functions can be either dict or proto
+        raw_data
+            Raw payload, a dictionary representing the json request or the raw grpc proto
 
         Returns
         -------
@@ -353,6 +358,8 @@ class SeldonClient:
             http_path=http_path,
             meta=meta,
             client_return_type=client_return_type,
+            raw_data=raw_data,
+            ssl=ssl,
         )
         self._validate_args(**k)
         if k["gateway"] == "ambassador" or k["gateway"] == "istio":
@@ -390,6 +397,7 @@ class SeldonClient:
         gateway_prefix: str = None,
         client_return_type: str = None,
         raw_request: dict = None,
+        ssl: bool = None,
     ) -> SeldonClientFeedback:
         """
 
@@ -443,6 +451,7 @@ class SeldonClient:
             gateway_prefix=gateway_prefix,
             client_return_type=client_return_type,
             raw_request=raw_request,
+            ssl=ssl,
         )
         self._validate_args(**k)
         if k["gateway"] == "ambassador" or k["gateway"] == "istio":
@@ -505,6 +514,7 @@ class SeldonClient:
         http_path: str = None,
         client_return_type: str = None,
         predictor: str = None,
+        ssl: bool = None,
     ) -> Dict:
         """
 
@@ -571,6 +581,7 @@ class SeldonClient:
             http_path=http_path,
             client_return_type=client_return_type,
             predictor=predictor,
+            ssl=ssl,
         )
         self._validate_args(**k)
         if k["gateway"] == "ambassador" or k["gateway"] == "istio":
@@ -1196,6 +1207,8 @@ def rest_predict_seldon(
     json_data: Union[str, List, Dict] = None,
     names: Iterable[str] = None,
     client_return_type: str = "proto",
+    raw_data: Dict = None,
+    meta: Dict = {},
     **kwargs,
 ) -> SeldonClientPrediction:
     """
@@ -1221,6 +1234,10 @@ def rest_predict_seldon(
        column names
     client_return_type
         the return type of all functions can be either dict or proto
+    raw_data
+        Raw payload (dictionary) given by the user
+    meta
+        Custom meta data map, supplied as tags
     kwargs
 
     Returns
@@ -1228,22 +1245,28 @@ def rest_predict_seldon(
        Seldon Client Prediction
 
     """
-    if bin_data is not None:
-        request = prediction_pb2.SeldonMessage(binData=bin_data)
-    elif str_data is not None:
-        request = prediction_pb2.SeldonMessage(strData=str_data)
-    elif json_data is not None:
-        request = json_to_seldon_message({"jsonData": json_data})
+    metaKV = prediction_pb2.Meta()
+    metaJson = {"tags": meta}
+    json_format.ParseDict(metaJson, metaKV)
+    if raw_data:
+        request = json_to_seldon_message(raw_data)
+        payload = raw_data
     else:
-        if data is None:
-            data = np.random.rand(*shape)
-        datadef = array_to_grpc_datadef(payload_type, data, names=names)
-        request = prediction_pb2.SeldonMessage(data=datadef)
-    payload = seldon_message_to_json(request)
+        if bin_data is not None:
+            request = prediction_pb2.SeldonMessage(binData=bin_data, meta=metaKV)
+        elif str_data is not None:
+            request = prediction_pb2.SeldonMessage(strData=str_data, meta=metaKV)
+        elif json_data is not None:
+            request = json_to_seldon_message({"jsonData": json_data})
+        else:
+            if data is None:
+                data = np.random.rand(*shape)
+            datadef = array_to_grpc_datadef(payload_type, data, names=names)
+            request = prediction_pb2.SeldonMessage(data=datadef, meta=metaKV)
+        payload = seldon_message_to_json(request)
 
     response_raw = requests.post(
-        "http://" + gateway_endpoint + "/api/v0.1/predictions",
-        json=payload,
+        "http://" + gateway_endpoint + "/api/v0.1/predictions", json=payload
     )
     if response_raw.status_code == 200:
         success = True
@@ -1283,6 +1306,7 @@ def grpc_predict_seldon(
     grpc_max_receive_message_length: int = 4 * 1024 * 1024,
     names: Iterable[str] = None,
     client_return_type: str = "proto",
+    raw_data: Dict = None,
     **kwargs,
 ) -> SeldonClientPrediction:
     """
@@ -1314,6 +1338,8 @@ def grpc_predict_seldon(
        Column names
     client_return_type
         the return type of all functions can be either dict or proto
+    raw_data
+        Raw payload (dictionary or proto) given by the user
     kwargs
 
     Returns
@@ -1321,19 +1347,24 @@ def grpc_predict_seldon(
        A SeldonMessage proto
 
     """
-    if bin_data is not None:
-        request = prediction_pb2.SeldonMessage(binData=bin_data)
-    elif str_data is not None:
-        request = prediction_pb2.SeldonMessage(strData=str_data)
-    elif json_data is not None:
-        request = json_to_seldon_message({"jsonData": json_data})
-    elif custom_data is not None:
-        request = prediction_pb2.SeldonMessage(customData=custom_data)
+    if isinstance(raw_data, prediction_pb2.SeldonMessage):
+        request = raw_data
+    elif raw_data:
+        request = json_to_seldon_message(raw_data)
     else:
-        if data is None:
-            data = np.random.rand(*shape)
-        datadef = array_to_grpc_datadef(payload_type, data, names=names)
-        request = prediction_pb2.SeldonMessage(data=datadef)
+        if bin_data is not None:
+            request = prediction_pb2.SeldonMessage(binData=bin_data)
+        elif str_data is not None:
+            request = prediction_pb2.SeldonMessage(strData=str_data)
+        elif json_data is not None:
+            request = json_to_seldon_message({"jsonData": json_data})
+        elif custom_data is not None:
+            request = prediction_pb2.SeldonMessage(customData=custom_data)
+        else:
+            if data is None:
+                data = np.random.rand(*shape)
+            datadef = array_to_grpc_datadef(payload_type, data, names=names)
+            request = prediction_pb2.SeldonMessage(data=datadef)
 
     channel = grpc.insecure_channel(
         gateway_endpoint,
@@ -1375,6 +1406,8 @@ def rest_predict_gateway(
     http_path: str = None,
     meta: Dict = {},
     client_return_type: str = "proto",
+    raw_data: Dict = None,
+    ssl: bool = None,
     **kwargs,
 ) -> SeldonClientPrediction:
     """
@@ -1416,6 +1449,8 @@ def rest_predict_gateway(
        Custom meta map
     client_return_type
         the return type of all functions can be either dict or proto
+    raw_data
+        Raw payload (dictionary) given by the user
 
     Returns
     -------
@@ -1426,31 +1461,34 @@ def rest_predict_gateway(
     metaKV = prediction_pb2.Meta()
     metaJson = {"tags": meta}
     json_format.ParseDict(metaJson, metaKV)
-
-    if bin_data is not None:
-        request = prediction_pb2.SeldonMessage(binData=bin_data, meta=metaKV)
-    elif str_data is not None:
-        request = prediction_pb2.SeldonMessage(strData=str_data, meta=metaKV)
-    elif json_data is not None:
-        request = json_to_seldon_message({"jsonData": json_data})
+    if raw_data is not None:
+        request = json_to_seldon_message(raw_data)
+        payload = raw_data
     else:
-        if data is None:
-            data = np.random.rand(*shape)
-        datadef = array_to_grpc_datadef(payload_type, data, names=names)
-        request = prediction_pb2.SeldonMessage(data=datadef, meta=metaKV)
-    payload = seldon_message_to_json(request)
+        if bin_data is not None:
+            request = prediction_pb2.SeldonMessage(binData=bin_data, meta=metaKV)
+        elif str_data is not None:
+            request = prediction_pb2.SeldonMessage(strData=str_data, meta=metaKV)
+        elif json_data is not None:
+            request = json_to_seldon_message({"jsonData": json_data})
+        else:
+            if data is None:
+                data = np.random.rand(*shape)
+            datadef = array_to_grpc_datadef(payload_type, data, names=names)
+            request = prediction_pb2.SeldonMessage(data=datadef, meta=metaKV)
+        payload = seldon_message_to_json(request)
 
     if not headers is None:
         req_headers = headers.copy()
     else:
         req_headers = {}
-    if call_credentials is None:
+    if call_credentials is None or ssl is False:
         scheme = "http"
     else:
         scheme = "https"
-        if not call_credentials is None:
-            if not call_credentials.token is None:
-                req_headers["X-Auth-Token"] = call_credentials.token
+    if not call_credentials is None:
+        if not call_credentials.token is None:
+            req_headers["X-Auth-Token"] = call_credentials.token
     if http_path is not None:
         url = url = (
             scheme
@@ -1553,6 +1591,7 @@ def explain_predict_gateway(
     http_path: str = None,
     client_return_type: str = "dict",
     predictor: str = None,
+    ssl: bool = None,
     **kwargs,
 ) -> SeldonClientPrediction:
     """
@@ -1622,13 +1661,13 @@ def explain_predict_gateway(
         req_headers = headers.copy()
     else:
         req_headers = {}
-    if channel_credentials is None:
+    if channel_credentials is None or ssl is False:
         scheme = "http"
     else:
         scheme = "https"
-        if not call_credentials is None:
-            if not call_credentials.token is None:
-                req_headers["X-Auth-Token"] = call_credentials.token
+    if not call_credentials is None:
+        if not call_credentials.token is None:
+            req_headers["X-Auth-Token"] = call_credentials.token
     if http_path is not None:
         url = (
             scheme
@@ -1733,6 +1772,8 @@ def grpc_predict_gateway(
     channel_credentials: SeldonChannelCredentials = None,
     meta: Dict = {},
     client_return_type: str = "proto",
+    raw_data: Dict = None,
+    ssl: bool = None,
     **kwargs,
 ) -> SeldonClientPrediction:
     """
@@ -1773,10 +1814,11 @@ def grpc_predict_gateway(
     channel_credentials
        Channel credentials - see SeldonChannelCredentials
     meta
-       Custom meta data map
+       Custom meta data map, supplied as tags
     client_return_type
         the return type of all functions can be either dict or proto
-
+    raw_data
+        Raw payload (dictionary or proto) given by the user
 
     Returns
     -------
@@ -1789,19 +1831,24 @@ def grpc_predict_gateway(
     metaJson = {"tags": meta}
     json_format.ParseDict(metaJson, metaKV)
 
-    if bin_data is not None:
-        request = prediction_pb2.SeldonMessage(binData=bin_data, meta=metaKV)
-    elif str_data is not None:
-        request = prediction_pb2.SeldonMessage(strData=str_data, meta=metaKV)
-    elif json_data is not None:
-        request = json_to_seldon_message({"jsonData": json_data})
-    elif custom_data is not None:
-        request = prediction_pb2.SeldonMessage(customData=custom_data, meta=metaKV)
+    if isinstance(raw_data, prediction_pb2.SeldonMessage):
+        request = raw_data
+    elif raw_data is not None:
+        request = json_to_seldon_message(raw_data)
     else:
-        if data is None:
-            data = np.random.rand(*shape)
-        datadef = array_to_grpc_datadef(payload_type, data, names=names)
-        request = prediction_pb2.SeldonMessage(data=datadef, meta=metaKV)
+        if bin_data is not None:
+            request = prediction_pb2.SeldonMessage(binData=bin_data, meta=metaKV)
+        elif str_data is not None:
+            request = prediction_pb2.SeldonMessage(strData=str_data, meta=metaKV)
+        elif json_data is not None:
+            request = json_to_seldon_message({"jsonData": json_data})
+        elif custom_data is not None:
+            request = prediction_pb2.SeldonMessage(customData=custom_data, meta=metaKV)
+        else:
+            if data is None:
+                data = np.random.rand(*shape)
+            datadef = array_to_grpc_datadef(payload_type, data, names=names)
+            request = prediction_pb2.SeldonMessage(data=datadef, meta=metaKV)
     options = [
         ("grpc.max_send_message_length", grpc_max_send_message_length),
         ("grpc.max_receive_message_length", grpc_max_receive_message_length),
@@ -1834,7 +1881,10 @@ def grpc_predict_gateway(
             )
         # This piece also allows for blank SSL Channel credentials in case this is required
         else:
-            grpc_channel_credentials = grpc.ssl_channel_credentials()
+            if ssl is False:
+                grpc_channel_credentials = grpc.local_channel_credentials()
+            else:
+                grpc_channel_credentials = grpc.ssl_channel_credentials()
         if channel_credentials.verify == False:
             # If Verify is set to false then we add the SSL Target Name Override option
             options += [
@@ -1922,8 +1972,7 @@ def rest_feedback_seldon(
         payload = feedback_to_json(request)
 
     response_raw = requests.post(
-        "http://" + gateway_endpoint + "/api/v1.0/feedback",
-        json=payload,
+        "http://" + gateway_endpoint + "/api/v1.0/feedback", json=payload
     )
     if response_raw.status_code == 200:
         success = True
