@@ -1,19 +1,19 @@
+import base64
 import json
 import logging
+from typing import Dict, List, Union
+
 import numpy as np
 from google.protobuf import json_format
-import base64
 
-from seldon_core.wrapper import get_rest_microservice, SeldonModelGRPC, get_grpc_server
 from seldon_core.metrics import SeldonMetrics
 from seldon_core.proto import prediction_pb2
-from seldon_core.utils import seldon_message_to_json
 from seldon_core.user_model import SeldonComponent
-
-from typing import List, Dict, Union
+from seldon_core.utils import seldon_message_to_json
+from seldon_core.wrapper import SeldonModelGRPC, get_grpc_server, get_rest_microservice
 
 
-class UserObject(object):
+class UserObject:
     def __init__(self, metrics_ok=True, ret_nparray=False):
         self.metrics_ok = metrics_ok
         self.ret_nparray = ret_nparray
@@ -37,7 +37,7 @@ class UserObject(object):
             return [{"type": "BAD", "key": "mycounter", "value": 1}]
 
 
-class UserObjectLowLevel(object):
+class UserObjectLowLevel:
     def __init__(self, metrics_ok=True, ret_nparray=False):
         self.metrics_ok = metrics_ok
         self.ret_nparray = ret_nparray
@@ -63,7 +63,7 @@ class UserObjectLowLevel(object):
             return seldon_message_to_json(response)
 
 
-class UserObjectLowLevelGrpc(object):
+class UserObjectLowLevelGrpc:
     def __init__(self, metrics_ok=True, ret_nparray=False):
         self.metrics_ok = metrics_ok
         self.ret_nparray = ret_nparray
@@ -78,7 +78,7 @@ class UserObjectLowLevelGrpc(object):
         return request
 
 
-class UserObjectBad(object):
+class UserObjectBad:
     pass
 
 
@@ -483,7 +483,7 @@ def test_unimplemented_aggregate_raw_on_seldon_component():
 
 
 def test_unimplemented_aggregate_raw():
-    class CustomObject(object):
+    class CustomObject:
         def aggregate(self, Xs, features_names):
             return sum(Xs) * 2
 
