@@ -1,18 +1,16 @@
-import logging
-import yaml
 import json
-import pytest
-
+import logging
 from unittest.mock import patch
 
-from seldon_core.metrics import SeldonMetrics
-from seldon_core.proto import prediction_pb2
-from seldon_core.wrapper import get_rest_microservice, SeldonModelGRPC
-from seldon_core.metadata import SeldonInvalidMetadataError, validate_model_metadata
-from seldon_core.utils import json_to_seldon_model_metadata
-
+import pytest
+import yaml
 from google.protobuf import json_format
 
+from seldon_core.metadata import SeldonInvalidMetadataError, validate_model_metadata
+from seldon_core.metrics import SeldonMetrics
+from seldon_core.proto import prediction_pb2
+from seldon_core.utils import json_to_seldon_model_metadata
+from seldon_core.wrapper import SeldonModelGRPC, get_rest_microservice
 
 # test default values
 
@@ -25,6 +23,7 @@ def test_default_values():
         "platform": "",
         "inputs": [],
         "outputs": [],
+        "custom": {},
     }
 
 
@@ -37,6 +36,7 @@ def test_default_values_with_env():
         "platform": "",
         "inputs": [],
         "outputs": [],
+        "custom": {},
     }
 
 
@@ -49,6 +49,7 @@ def test_default_values_with_colon_in_env():
         "platform": "",
         "inputs": [],
         "outputs": [],
+        "custom": {},
     }
 
 
@@ -68,6 +69,8 @@ def test_v1_only_messagetype(messagetype):
         - messagetype: {messagetype}
         outputs:
         - messagetype: {messagetype}
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -77,6 +80,7 @@ def test_v1_only_messagetype(messagetype):
         "platform": "seldon",
         "inputs": [{"messagetype": f"{messagetype}"}],
         "outputs": [{"messagetype": f"{messagetype}"}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -84,6 +88,7 @@ def test_v1_only_messagetype(messagetype):
         "platform": "seldon",
         "inputs": [{"messagetype": f"{messagetype}"}],
         "outputs": [{"messagetype": f"{messagetype}"}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -100,6 +105,8 @@ def test_v1_mixed_multiple_inputs():
         - messagetype: jsonData
         outputs:
         - messagetype: "binData"
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -115,6 +122,7 @@ def test_v1_mixed_multiple_inputs():
             {"messagetype": "jsonData"},
         ],
         "outputs": [{"messagetype": "binData"}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -128,6 +136,7 @@ def test_v1_mixed_multiple_inputs():
             {"messagetype": "jsonData"},
         ],
         "outputs": [{"messagetype": "binData"}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -148,6 +157,8 @@ def test_v1_array(messagetype):
         - messagetype: {messagetype}
           schema:
             shape: [ 1 ]
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -162,6 +173,7 @@ def test_v1_array(messagetype):
             }
         ],
         "outputs": [{"messagetype": f"{messagetype}", "schema": {"shape": [1]}}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -174,6 +186,7 @@ def test_v1_array(messagetype):
             }
         ],
         "outputs": [{"messagetype": f"{messagetype}", "schema": {"shape": [1.0]}}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -198,6 +211,8 @@ def test_v1_json_with_schema():
                         format: double
         outputs:
         - messagetype: jsonData
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -221,6 +236,7 @@ def test_v1_json_with_schema():
             }
         ],
         "outputs": [{"messagetype": "jsonData"}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -242,6 +258,7 @@ def test_v1_json_with_schema():
             }
         ],
         "outputs": [{"messagetype": "jsonData"}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -312,6 +329,8 @@ def test_v2():
         - datatype: BYTES
           name: output
           shape: [ 3 ]
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -321,6 +340,7 @@ def test_v2():
         "platform": "seldon",
         "inputs": [{"datatype": "BYTES", "name": "input", "shape": [1, 4]}],
         "outputs": [{"datatype": "BYTES", "name": "output", "shape": [3]}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -328,6 +348,7 @@ def test_v2():
         "platform": "seldon",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": ["1", "4"]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": ["3"]}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -348,6 +369,8 @@ def test_v1_v2_mix():
         - datatype: BYTES
           name: output
           shape: [ 3 ]
+        custom:
+          tag-key: tag-value
     """
     meta_json = validate_model_metadata(yaml.safe_load(data))
     meta_proto = json_to_seldon_model_metadata(meta_json)
@@ -360,6 +383,7 @@ def test_v1_v2_mix():
             {"messagetype": "jsonData"},
         ],
         "outputs": [{"datatype": "BYTES", "name": "output", "shape": [3]}],
+        "custom": {"tag-key": "tag-value"},
     }
     assert json.loads(json_format.MessageToJson(meta_proto)) == {
         "name": "my-model-name",
@@ -370,6 +394,7 @@ def test_v1_v2_mix():
             {"messagetype": "jsonData"},
         ],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": ["3"]}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
@@ -383,6 +408,7 @@ def test_validate_model_metadata():
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1]}],
+        "custom": {"tag-key": "tag-value"},
     }
     with patch("seldon_core.metadata.MODEL_IMAGE", None):
         assert meta == validate_model_metadata(meta)
@@ -395,6 +421,7 @@ def test_validate_model_metadata_with_env():
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1]}],
+        "custom": {"tag-key": "tag-value"},
     }
     with patch("seldon_core.metadata.MODEL_IMAGE", "seldonio/sklearn-iris:0.1"):
         assert meta == validate_model_metadata(meta)
@@ -407,6 +434,7 @@ def test_validate_model_metadata_with_colon_in_env():
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1]}],
+        "custom": {"tag-key": "tag-value"},
     }
     with patch("seldon_core.metadata.MODEL_IMAGE", "localhost:32000/sklearn-iris:0.1"):
         assert meta == validate_model_metadata(meta)
@@ -450,6 +478,8 @@ outputs:
 - datatype: BYTES
   name: output
   shape: [ 1, 2, 3 ]
+custom:
+  tag-key: tag-value
 """
 
 
@@ -460,6 +490,7 @@ json_meta = """
     "platform": "seldon",
     "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1, 2]}],
     "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1, 2, 3]}],
+    "custom": {"tag-key": "tag-value"},
 }
 """
 
@@ -471,6 +502,7 @@ class UserObject:
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1]}],
+        "custom": {"tag-key": "tag-value"},
     }
 
     def predict(self, X, features_names):
@@ -509,6 +541,7 @@ def test_model_metadata_ok_grpc():
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": ["1"]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": ["1"]}],
+        "custom": {"tag-key": "tag-value"},
     }
 
 
